***********************************************************************
***************************** BROWSER  ********************************
***********************************************************************
PROCEDURE browser
*
* No application is complete without a full scale browsing facility.
* This browser permits one to rapidly search a file using either
* keystrokes or the mouse.  Mouse movements are interpretted permitting
* smooth scrolling as well as buttons that perform special functions.
*
* Searchs are incremental on a character by character basis.  The user
* moves to a field and starts typing.  As the typing procedes, the browser
* will search and locate the desired string.
*
* To aid users on slower computers, there is a keyboard shortcut (uses the
* the ASK utility) for the user to type an entire string at one time.
* This permits the browser to search for the entire string without using
* time to incrementally search for each character.
*
* The browser has a subsetting capability, where the user can specify any
* level of search options to any depth.  The display is successfully reduced
* by the user's request.  Requests are placed on a stack, permitting the
* user to specify to reverse the subset to re-view the larger selection
* of data.
*
* Lack of data will display a message.  If requested, the user will be
* given the option to add data to the file, using a programmer supplied
* entry procedure.  If the add procedure is specified, the user will be
* given a button to press to add data when desired.  The label on the
* 'add' button can be changed, permitting apps to perform other
* functions (for example - dial a telephone).
*
* The user is given the option of selecting a single record, or to select
* multiple records at one time.  If multiple records are to be selected,
* the database file must have a field called 'SELECTED'.  This field must
* be a logical value.
*
* Color schemes are matched using the standard PRO tools pro_color options.
*
* Help is activated with a rightmouse or an F1 keypress.
* On help, the topic will be set to the default from pro_cfg [31]
* (see PINSTALL.PRG), or by the content of the variable: pro_b_help.
*
* Ordinarily GO TOP should be issued before entering the browser.
* Browser does not automatically position to the top in order to give
* the programmer the opportunity to position to a desired record.
*
* Automatically uses SEEK for single fields that are already defined
* in a SET ORDER instruction.  Uses LOCATE for all other searches.
*
* NOTE: For efficient searches, indexes should be defined as UPPER (field).
* Standard searches will employ the index when possible.
* If the index is NOT defined as UPPER (field), then failures to locate
* an item may happen because data is converted to upper case before 
* a search is begun.
*
*	Push button options include:
*		esc 	- escape
*		F2  	- Accept
*		F5		- Find dialog box
*		Alt+F5	- Find Again
*		F6		- Subset dialog box
*		Alt_F6	- Superset
*   The developer may select the labels for the buttons
***********************************************************************

PARAMETERS lp_title, lp_fields, selrec, insert_proc , ;
			lp_for , lp_param , lp_nobutton , lp_window , lp_notes
* lp_title is title of browse window.  Default to name of current database.
* lp_fields is a list of fields to browse.  This follows the conventions
*		   for a browse field list EXCEPT that each field must be 
*		   separated by a '|'.  Defaults to all fields in database.
* selrec enables the select multiple record mode.  Default .F.
*			The database must have a field called 'selected' to use this 
*			function.  'Selected' must be a logical field.  This field is
*			also typically indexed to aid the calling program to find
*			items that have been selected, and to improve performance
*			while using this browser.
* insert_proc enables selection of a new record w/returning that record
*            as the result.  If no records, will automatically execute
* 			 the insert command after requesting if it is OK.  Default .F.
* lp_for	  - FOR clause - use a text expression
* lp_param - additional parameters to include w/the browse
* lp_nobutton -	Do not put pushbuttons at the bottom of the screen.
* lp_window - Application defined name of window for browse.  
*			If .T. or if the window does not already exist, browser will 
*			create a full screen browser.  Browser will NOT release user 
*			defined windows used in a browse.  However, to keep the screen 
*			neat, these windows will be hidden when Browser is finished.
* lp_notes	Special note information box
*
* Examples:  
*
*	1.		DO browser
*
*			The above will create a default browse using all fields.
*
*			The user may use the keyboard or the mouse to browse through
*			the file.  When the desired entry is located, pressing the
*			mouse, enter key, or F2 will select the record.  Browser will
*			stop, returning control to the running program.  The file
*			will be positioned to the record the user selected.
*
*			If the user escaped, the file will be returned to be pointing
*			at the same record that was present before the browser
*			was run.
*
*			Options will be presented through buttons at the bottom of the
*			screen:  Cancel/Accept , Search/Again , Subset/Super.
*
*			Cancel/Accept - additional ways to visibly inform the
*			user how to terminate the browse.
*
*			Search/Again - pop up a dialog box for the user to enter
*			a search string rather than incrementally searching for a 
*			character at a time.
*
*			Subset/Superset - pop up a dialog box to create a subselection
*			of records to be viewed.			
*
*	2.		x = browser ('This is a test' , 'field1|field2')
*
*			Same as example 1 PLUS:
*			
*			The above will create a browse with a title of 'This is a Test'.
*			The browse will display 2 fields: Field1 and Field2.
*
*			When done, x will be filled with a .T. if the user pressed
*			the accept button or selected an entry.  x will be filled with
*			.F if the user canceled.
*
*	3.		SET ORDER TO field2
*			x = browser ('This is a test' , 'field1|field2')
*
*			Same as example 2 PLUS:
*
*			Database will be sorted by field2.
*			Incremental searches on field2 will be indexed.  This will
*			usually result in much faster searches.
*
*	4.		x = browser ('This is a test' , 'field1|field2' , .T.)
*
*			Same as example 2 PLUS:
*
*			The left column will be filled with the SELECTED field.
*			If the user presses ENTER or double-clicks on an entry,
*			the select box will indicate that entry was selected.
*
*			The application calling browser may pre-set the SELECTED field.			
*			
*			When done, the application may search for the SELECTED field
*			being set to .T. to determine records that the user selected.
*
*			CAUTION:  The programmer is responsible to ensure that there
*			is a field with the name: SELECTED.  This field MUST BE TYPE
*			LOGICAL.
*
*			Note: It is usually to a performance benefit if this field
*			is indexed.
*
*
*	5.		Adding Records
*
*			a. Using a local variable to indicate the user
*			   wants to add a record
*
*				l_add = .F.
*				IF !browser ('Select an Entry' , 'Code1 | Code2 | Code3' , ;
*					.F. , 'l_add = .T.'
*					RETURN
*				ENDIF
*				IF l_add
*					WAIT "We really wanted to add a record here" WINDOW
*				ENDIF
*
*			b. Using a callable procedure when the user wants to add
*			   a record.  In this case, the browse will resume after
*				the user adds the record.
*
*				l_add = .F.		&& Used to detect that record was added
*				IF !browser ('Select an Entry' , 'Code1 | Code2 | Code3' , ;
*					.F. , 'l_add = p_add()'
*					RETURN
*				ENDIF
*
*				PROCEDURE p_add
*					WAIT 'Add record source goes here' WINDOW
*				RETURN .T.	&& Let calling app know we added something
*
*
*	6.		Limit display to a subset of a file
*
*			IF !browser ("Select a Facility" , ;
*							"fac_code :H='Facility Code' | " + ;
*							"fac_name :H='Facility'      | " + ;
*							"mast_numb :H='Main Phone'" , ;
*							.T. , "=Add_Fac()" , ;
*							"UPPER (fac_code) = 'A'" 
*				RETURN
*			ENDIF
*
*			Note the use of standard browse field naming conventions
*			to re-label column headings.
*
*			Only the selection of records where the UPPER (fac_code) = 'A'
*			will be displayed.
*
*	7.		Use of special configuration functions
*
*			pro_browse = '156'
*			=browser()
*		
*			Special options are invoked:
*				1 - If only a single record is in the file, that 
*					record is automatically selected.
*				5 - The options for search/again are disabled.
*					Those buttons are not displayed.
*				6 - The options for subset/superset are disabled.
*					Those buttons are not displayed.
*
*	8.		Re-label of the 'Add' push button
*
*			The following example will illustrate use of the 'Add'
*			push button for other capabilities (modify records, dialers ...)
*
*			DIMENSION pro_brow_but [8]
*			pro_brow_but [1] = 'Update'
*			pro_brow_but [2] = 'Done'
*			pro_brow_but [7] = 'Dial'
*			pro_browse   = 'PT'
*			l_opt = browser('Choose Numbers to Dial' , .F. , .F. , ;
*						'wait window "Dialling"')
*			RELEASE MEMORY LIKE 'pro_brow_but*'
*			IF l_opt
*				&& Update
*			ELSE
*				&& Done
*			ENDIF
*
*			Pressing ENTER or using mouse and clicking at a record will
*			NOT terminate the browse.
*
*			Pressing the INS or using mouse on what is normally the
*			add button will cause the message 'Dialling' to be displayed.
*			At this point one can use a phone dialler ... or whatever.
*
*			The browse will not be terminated and re-started when the add
*			button is pressed.
*
*			The 'Accept', 'Cancel', and 'Add' buttons are re-labeled.
*
*	9.	Browser without the buttons at the bottom of the screen
*
*		=browser('Title' , .f. , .f. , .f. , .f. , .f. , .T.)
*
*		This is a full screen incremental browser.
*		All push button options are available through the function keys.
*
* 	10. Browser with your own window definitions.
*
*		DEFINE WINDOW win_test ;
*			FROM 5,20 to 15,60 ;
*			COLOR SCHEME 10 ;
*			CLOSE FLOAT
*		=browser('Title' , .f. , .f. , .f.,.f.,.f.,.F.,'win_test')
*
*		A small browsing screen will be presneted.  Push button options
*		are displayed at the bottom of the screen.
*
* 	11. Browser with your own window definitions.  No push buttons.
*
*		DEFINE WINDOW win_test ;
*			FROM 5,20 to 15,60 ;
*			COLOR SCHEME 10 ;
*			CLOSE FLOAT
*		=browser('Title' , .f. , .f. , .f.,.f.,.f.,.t.,'win_test')
*
*		A small browsing screen will be presented.  Push button options
*		are not displayed at the bottom of the screen.  Use function keys
*		to activate the special functions.
*
*-----------------------------------------------------------------------
* (c) Creative Software Solutions, 1993, 1994. All rights reserved.   
*
* This application is part of the PRO TOOLS collection.               
* For information or purchase:  Call (612) 546-0527.                  
*
* Written by Ira D. Wald.                                             
*
*-----------------------------------------------------------------------
PRIVATE ALL LIKE 'lp_*' , ALL LIKE 'la_*'

SET TALK OFF		&& No echo to the screen
pro_ret = .F.		&& Return code indicating a failure.
					&& Code is only set to true if a correct return
					&& value is selected.
					&& File will be returned to it's original record
					&& position if an escape is pressed or if a
					&& failure occurs.

* Default setting for general browser functionality
*	Local Variable pro_browse
*			''	Normal browse options
*			'S'	Require an F2 or Accept before
*				using a record.  This uses part
*				of the multiple record select
*				process, hence there is a need
*				for a field named 'SELECTED' in
*				the record.
*				Browser will automatically set all items in SELECTED
*				to .F. prior to displaying a screen.
*				Note that this may be slower than a 'normal' browse
*				display - especially if used for larger files.
*			'P' Permit the ADD function to be run from the active
*				browse (normally the browse will terminate and restart
*				itself when the ADD funtion is called).
*			'T'	Do NOT terminate unless the F2 key is pressed.
*				Other selection keys will cause the browse to 
*				leave and come back again.
*			'0' If it is OK for no records to be selected when multiple
*				selections option is set (SELREC parameter).
*			'1' If there is just one record that would be displayed,
*				that record will be automatically selected.
*			'5' Disable use of F5/Alt-F5 for search
*			'6' Disable use of F6/Alt-F6 for subsetting
pro_browse = IIF (TYPE ('pro_browse') = 'C' , pro_browse , xz_pro_cfg [08])

*	Help Title for browse screen: pro_b_help (activated when F1 is pressed)
*
*	Label for pushbuttons 
*
DIMENSION pro_brow_but [8]

IF EOF() AND !BOF()	&& Check where we are starting from in the file
	SKIP -1
ENDIF
lp_start_rec = RECNO()
lp_start_rec = IIF (lp_start_rec = 1 AND EOF() ;
				OR EOF() AND BOF() , 0 , lp_start_rec)

lp_window = IIF (TYPE ('lp_window') = 'C' , lp_window , 'pro_brow')
lp_wind_browse = ''	&& Holding place for actual browse window name

*	Default color schemes
lp_color = IIF (TYPE ('pro_color') = 'N' , pro_color , 10)

*	Control typeahead to help with timing with keystrokes
lp_typeahead = SET ('TYPEAHEAD')
SET TYPEAHEAD TO 0

* Set up capability for the user to perform their own super/sub-sets
* These are all limited to the FOR specification supplied on the
* browser procedure call.
DIMENSION lp_subset [1]
lp_subset_cnt = 1
lp_subset [1] = IIF (TYPE ('lp_for') = 'C' , 'FOR (' + lp_for + ')' , '' )

* Set up the working environment
=set_push()

SET RESOURCE OFF
SET ESCAPE OFF
SET EXACT OFF
SET NEAR ON
SET STATUS BAR ON

* 	Set Defaults
lp_title  = IIF (EMPTY (lp_title) , ALIAS() , lp_title)
lp_param  = IIF (TYPE ('lp_param') = 'L' , '' , lp_param)

*  Info re: the current field being processed
lp_cur_fld = ''	&& field name of the current field
lp_cur_type= ''	&& Type of the current field
lp_cur_len = 0  && Length of the current field

* If the user does not specify fields, collect info for each field
IF EMPTY (lp_fields)
	lp_len = AFIELDS (lp_field_a)
	lp_fields = TRIM (lp_field_a [1,1])
	FOR lp_cnt = 2 TO lp_len
		lp_fields = lp_fields + '|' + TRIM (lp_field_a [lp_cnt,1])
	ENDFOR
ENDIF

* Add a WHEN clause to each field permitting browser to know 
* when movement occurs between fields.
lp_fields = b_insert_when (lp_fields)

*  Add 'selected' field when multiple occurances are to be selected
*  or if pro_browse indicates that all selections must be via an F2
*  rather than an enter (option 'S').
lp_check = IIF (_DOS , '' , 'X')
lp_fields = IIF (selrec OR (AT ('S' , pro_browse) > 0) , ;
			"bx=IIF(selected,lp_check,' '):h='" + lp_check + "': w=.F.," + lp_fields , ;
			lp_fields)

* For single select (not auto return) option, set all selected items to .F.
IF AT ('S' , pro_browse) > 0 AND !selrec AND lp_start_rec > 0
	REPLACE selected WITH .F. FOR selected
	lp_rec_sel = 1
	GOTO lp_start_rec
ENDIF

lp_butt = 0

* Set parameter defaults
lp_locate = ""   && .t. if there is a locate scope active  
lp_seek   = ""
lp_found  = .F.	&& .t. if last locate/seek successful
lp_indx_search = .F.	&& Do an indexed search
curKey   = ""	&& Keep track of last key entered
addrecrd = .F.	&& Flag to call a user add record procedure
lp_click  = 0	&& Last time a left mouse click issued.
				&& Used to test if we are processing a double click.

* Help Setup - determine the Default help screen
pro_b_help = IIF (TYPE ('pro_b_help') = 'C' , pro_b_help , xz_pro_cfg [31])
lp_b_hlpx = pro_b_help
lp_b_hlpy = SET ('topic')
lp_b_hlpz = xz_pro_def [49]

* Identify the current indexed field
lp_indx    = IIF (SYS(21) = "0"	, '' , UPPER (SYS (14, VAL( SYS(21)))))
IF (AT ('UPPE' , lp_indx) > 0)		&& Upper case index
	lp_indx = STRTRAN (lp_indx , 'UPPER', '')
	lp_indx = STRTRAN (lp_indx , 'UPPE' , '')
	lp_indx = STRTRAN (lp_indx , '(' , '')
	lp_indx = STRTRAN (lp_indx , ')' , '')
ENDIF

*	Initialize sizing factors for scroll bar controls under Mac and Windows.
*	This is set in instruct().
lp_factx = 2
lp_facty = 2

*
* Set up an array of buttons to be placed on the screen
*
* Array will contain the name of the button and coordinates (calculated by butt_pos())
*
*	Array consists of:
*		la_butt [n,1] = name
*		la_butt [n,2] = procedure to run if button is hit
*		la_butt [n,3] = starting x coord
*		la_butt [n,4] = starting y coord
DIMENSION la_butt [8,4]
la_butt [1,1] = IIF (TYPE ('pro_brow_but [1]') = 'C' , pro_brow_but [1] , 'Accept') 
la_butt [1,2] = 'DO brow_F2'

la_butt [2,1] = IIF (TYPE ('pro_brow_but [2]') = 'C' , pro_brow_but [2] , 'Cancel')
la_butt [2,2] = 'DO brow_Esc'

la_butt [3,1] = IIF (TYPE ('pro_brow_but [3]') = 'C' , pro_brow_but [3] , 'Search')
la_butt [3,2] = 'DO brow_F5'

la_butt [4,1] = IIF (TYPE ('pro_brow_but [4]') = 'C' , pro_brow_but [4] , 'Again')
la_butt [4,2] = 'DO brow_altF5'

la_butt [5,1] = IIF (TYPE ('pro_brow_but [5]') = 'C' , pro_brow_but [5] , 'Sub-set')
la_butt [5,2] = 'DO brow_F6'

la_butt [6,1] = IIF (TYPE ('pro_brow_but [6]') = 'C' , pro_brow_but [6] , 'Super')
la_butt [6,2] = 'DO brow_altF6'

la_butt [7,1] = IIF (TYPE ('pro_brow_but [7]') = 'C' , pro_brow_but [7] , 'Add')
la_butt [7,2] = 'DO brow_insert'

la_butt [8,1] = IIF (TYPE ('pro_brow_but [8]') = 'C' , pro_brow_but [8] , 'Select')
la_butt [8,2] = 'DO brow_select'

* Force picker to ignore everything by setting coordinates to be unusable.
* instruct will call butt_pos to set the real coordinates for the buttons actually chosen
* to be used.
FOR la_cnt = 1 TO 8
	STORE -100 TO la_butt [la_cnt , 3] , la_butt [la_cnt , 4]
ENDFOR

* Determine dimensions of browser
* Locations of bottom of the screen message line
lp_notes_offset = IIF (TYPE ('lp_notes') = 'C' , 3 , 0)

lp_butt_cnt = 2 + IIF ('5' $ pro_browse , 0 , 2) + IIF ('6' $ pro_browse , 0 , 2) + ;
				IIF (EMPTY (insert_proc) , 0 , 1) + IIF (selrec , 1 , 0)
DO CASE
	CASE _DOS OR _UNIX
	lp_msg_cols = SCOLS() - 2
	lp_w_rows = SROWS()
	lp_msg_rows = 6
	lp_msg_factor = 1

	CASE _WINDOWS
	lp_msg_cols = SYSMETRIC (1)  ;
					 / FONTMETRIC (6 , xz_pro_def [11] , xz_pro_def [12] , xz_pro_def [13]) - 2
	lp_w_rows = SYSMETRIC (2)   ;
					 / FONTMETRIC (1 , xz_pro_def [11] , xz_pro_def [12] , xz_pro_def [13]) 
	lp_msg_rows=  xz_pro_def [14] * (CEILING (17 * lp_butt_cnt / (lp_msg_cols-2)) + 1)

	OTHERWISE
	lp_msg_cols = SYSMETRIC (1)  ;
					 / FONTMETRIC (6 , xz_pro_def [11] , xz_pro_def [12] , xz_pro_def [13]) - 2
	lp_w_rows = SYSMETRIC (2)   ;
					 / FONTMETRIC (1 , xz_pro_def [11] , xz_pro_def [12] , xz_pro_def [13]) 
	lp_msg_rows=  xz_pro_def [14] * (CEILING (17 * lp_butt_cnt / (lp_msg_cols-2)) + 1) 
ENDCASE

lp_message_row = SROWS() - IIF (_MAC OR _WINDOWS , 0.5 , 0)  - ;
					(lp_msg_rows + lp_notes_offset) * SROWS() / lp_w_rows
lp_scroll_col  = lp_msg_cols 

lp_brow_rec = lp_start_rec	&& Set up record to point to

lp_more = .T.				&& Loop until no more special events
DO WHILE lp_more
	STORE .F. TO addrecrd

	* Count Records to see if we should even try to browse
	DO CASE
		CASE lp_brow_rec = 0
			cntRecords = 0
		CASE EMPTY (lp_subset [lp_subset_cnt]) AND EMPTY (SET ('filter'))
			cntRecords = RECCOUNT()
		CASE lp_start_rec > 0
			COUNT TO cntRecords &lp_subset [lp_subset_cnt]
			GOTO lp_brow_rec
		OTHERWISE
			cntRecords = 0
	ENDCASE
	
	DO CASE
		CASE lp_subset_cnt = 1 AND cntRecords = 0
			lp_more = .F.
			IF EMPTY (insert_proc)
				pro_color = 7
				=choose_1 (RTRIM (lp_title) , "No records available.  Continue.")
				pro_ret = .F.
			ELSE
				pro_color = 7
				IF choose_h (RTRIM (lp_title) + ". No records to view." , ;
							"Enter a new record" , "Cancel Browse") 
					addrecrd = .T.
					pro_ret = .T.
				ELSE
					pro_ret = .F.
				ENDIF
			ENDIF
		
		CASE cntRecords = 0
			WAIT "No records to view" WINDOW NOWAIT
			lp_more = .T.
			pro_color = 7
			=choose_1 ("No records in this subset" , ;
					  "Return to previous extraction")
			lp_subset_cnt = lp_subset_cnt - 1
		
		* Do not browse if only one record and pro_browse contains a '1'
		CASE lp_subset_cnt = 1 AND cntRecords = 1 ;
		 AND AT ('1' , pro_browse)	> 0
			
			GOTO TOP		&& FInd the single record
			IF !EMPTY (lp_for)
				LOCATE FOR &lp_for	
			ENDIF
			lp_more = .F.	&& Leave browser
			pro_ret = .T.	&& Success w/the one entry
					
		OTHERWISE
		
			pro_b_help = lp_b_hlpx		&& Set up help for this browse
			pro_color = lp_color
			=pro_init (lp_title,10)		&& Standard setup process
			DO instruct   	&& screen instructions and buttons
			DO key_setup     && initializes keys

			IF !EMPTY (lp_subset [lp_subset_cnt])
				WAIT 'Subset: ' + LTRIM (STR (cntRecords)) + ' records' WINDOW NOWAIT
			ENDIF
			
			BROWSE FIELDS &lp_fields  WINDOW (lp_window) ;
					&lp_subset [lp_subset_cnt]  ;
					&lp_param  ;
					NOMODIFY  ;
					NODELETE  ;
					NOREFRESH ;
					NOMENU ;
					WHEN mouse_set()
			CLEAR TYPEAHEAD
			xz_pro_def [49] = lp_b_hlpz
			=pro_end()
			POP KEY
			
	ENDCASE

	IF addrecrd
		IF WEXIST (lp_window)
			HIDE WINDOW (lp_window)
		ENDIF
		IF WEXIST ('pro_stat')
			HIDE WINDOW pro_stat
		ENDIF
		lp_brow_rec = RECNO()	&& Remember record we are currently
								&& pointing to
		pro_ret = .T.
		&insert_proc
		*	If multi-record selection, go back to the selection window
		*	Select the record if pro_ret remains .T.
		IF selrec OR AT ('S' , pro_browse)	> 0
			lp_more = .T.
			IF pro_ret
				REPLACE selected WITH .T.		
				STORE RECNO() TO lp_start_rec , lp_brow_rec
			ENDIF
		ENDIF
	ENDIF

ENDDO
		
RELEASE WINDOWS pro_brow , pro_stat
IF WEXIST (lp_window)
	HIDE WINDOW (lp_window)
ENDIF

DO CASE
	CASE !pro_ret AND lp_start_rec > 0	&& Return to the original record
		GOTO lp_start_rec
	CASE AT ('S' , pro_browse) > 0 AND !selrec AND lp_rec_sel > 0
		GOTO lp_rec_sel
ENDCASE

=set_pop()				&& Restore settings
SET TOPIC TO &lp_b_hlpy
SET TYPEAHEAD TO lp_typeahead

RETURN pro_ret


**********************************************************************
PROCEDURE key_setup

CLEAR TYPEAHEAD
=brow_key_start()

* Reconfigure the mouse

ON KEY LABEL leftmouse DO process_mouse WITH MWINDOW() , MCOL('') , MROW('') , ;
										MCOL (MWINDOW()) , MROW(MWINDOW())

* Reconfigure the keyboard
ON KEY LABEL Enter 	DO brow_Enter
ON KEY LABEL Esc		DO brow_esc
ON KEY LABEL F2		DO brow_F2
ON KEY LABEL F5 		DO brow_F5
ON KEY LABEL ALT+F5	DO brow_altF5
ON KEY LABEL F6 		DO brow_F6
ON KEY LABEL ALT+F6	DO brow_altF6
ON KEY LABEL F9 		POP KEY ALL
ON KEY LABEL Home 	DO brow_home
ON KEY LABEL END 		DO brow_end
ON KEY LABEL Backspace 	DO brow_back

ON KEY LABEL A 		DO brow_key WITH  'A'
ON KEY LABEL B		DO brow_key WITH  'B'
ON KEY LABEL C		DO brow_key WITH  'C'
ON KEY LABEL D		DO brow_key WITH  'D'
ON KEY LABEL E		DO brow_key WITH  'E'
ON KEY LABEL F			DO brow_key WITH  'F'
ON KEY LABEL G		DO brow_key WITH  'G'
ON KEY LABEL H		DO brow_key WITH  'H'
ON KEY LABEL I			DO brow_key WITH 'I'
ON KEY LABEL J			DO brow_key WITH 'J'
ON KEY LABEL K		DO brow_key WITH 'K'
ON KEY LABEL L			DO brow_key WITH 'L'
ON KEY LABEL M		DO brow_key WITH 'M'
ON KEY LABEL N		DO brow_key WITH 'N'
ON KEY LABEL O		DO brow_key WITH 'O'
ON KEY LABEL P		DO brow_key WITH 'P'
ON KEY LABEL Q		DO brow_key WITH 'Q'
ON KEY LABEL R		DO brow_key WITH 'R'
ON KEY LABEL S		DO brow_key WITH 'S'
ON KEY LABEL T			DO brow_key WITH 'T'
ON KEY LABEL U		DO brow_key WITH 'U'
ON KEY LABEL V			DO brow_key WITH 'V'
ON KEY LABEL W		DO brow_key WITH 'W'
ON KEY LABEL X		DO brow_key WITH 'X'
ON KEY LABEL Y		DO brow_key WITH 'Y'
ON KEY LABEL Z		DO brow_key WITH 'Z'
ON KEY LABEL 1			DO brow_key WITH '1'
ON KEY LABEL 2			DO brow_key WITH '2'
ON KEY LABEL 3			DO brow_key WITH '3'
ON KEY LABEL 4			DO brow_key WITH '4'
ON KEY LABEL 5			DO brow_key WITH '5'
ON KEY LABEL 6			DO brow_key WITH '6'
ON KEY LABEL 7			DO brow_key WITH '7'
ON KEY LABEL 8			DO brow_key WITH '8'
ON KEY LABEL 9			DO brow_key WITH '9'
ON KEY LABEL 0			DO brow_key WITH '0'
ON KEY LABEL -			DO brow_key WITH '-'
ON KEY LABEL /			DO brow_key WITH '/'
ON KEY LABEL *			DO brow_key WITH '*'
ON KEY LABEL (			DO brow_key WITH '('
ON KEY LABEL )			DO brow_key WITH ')'
ON KEY LABEL '.'		DO brow_key WITH '.'
ON KEY LABEL ','		DO brow_key WITH ','
ON KEY LABEL '_'		DO brow_key WITH '_'
ON KEY LABEL Spacebar	DO brow_key WITH ' '

IF !EMPTY (insert_proc)
	ON KEY LABEL INS	DO brow_insert
ENDIF

CLEAR TYPEAHEAD

RETURN                  && return to calling program
**********************************************************************

PROCEDURE brow_key
PARAMETERS b_key

=brow_key_start()

lp_locate = ""
lp_seek = IIF (len(lp_seek) < lp_cur_len , ;
			lp_seek + b_key , b_key)
=findrec()
=brow_key_done()

*******************************************************************************************

PROCEDURE brow_F5
*
*	Process the F5 key - Search
*
=brow_key_start()

	lp_size = MIN (FSIZE (lp_cur_fld) , 60)
	lp_seek = ask ("Enter Search Criteria for [" + lp_cur_fld + "]" , ;
      					x2null(EVALUATE (lp_cur_fld)) , ;
      					lp_size , "@!" ,  "Accept" , "Cancel")
    IF pro_ret
		lp_seek = xlate2c (lp_seek)
       	=findrec()
    ELSE
      	lp_seek = ''
    ENDIF

=brow_key_done()

*******************************************************************************************

PROCEDURE brow_altF5
*
*	Process the alt_F5 key - Search Again
*
=brow_key_start()

IF LEN (lp_seek) = 0			&& Nothing else to search again - re-do search criteria
	DO brow_F5
ELSE
   	WAIT "  Searching ...  " WINDOW NOWAIT
	lp_recno = RECNO()
	lp_indx_tmp = lp_indx_search && Disable potential indexed search processing
	lp_indx_search = .F.

	IF LEN (lp_locate) = 0		&& Find first occurance
		DO findrec
	ENDIF

	CONTINUE					&& Find the next occurance

	IF !FOUND()                   		&& no match found
		GOTO lp_recno                 	&& go to previous record
		STORE "" to lp_locate , lp_seek 	&& deactivate locate scope
		WAIT "No More Matches.  Press any key to continue." WINDOW
	ENDIF
	WAIT CLEAR
	lp_indx_search = lp_indx_tmp
ENDIF

=brow_key_done()

*******************************************************************************************

PROCEDURE brow_F6
*
*	Process the F6 key - Subset View
*
=brow_key_start()

lp_temp = b_subset()
IF !EMPTY (lp_temp)
	lp_subset_cnt = lp_subset_cnt + 1
	DIMENSION lp_subset [lp_subset_cnt]
	lp_subset [lp_subset_cnt] = ;
	IIF (EMPTY (lp_subset [lp_subset_cnt - 1]) , ;
			'FOR ' + lp_temp , ;
			lp_subset [lp_subset_cnt - 1] + ' AND ' + lp_temp)
	lp_more = .T.
 	keyboard chr(23)		 && force ctrl-w to end
ENDIF

=brow_key_done()
  
*******************************************************************************************

PROCEDURE brow_altF6
*
*	Process the Alt+F6 key - Superset View
*
=brow_key_start()

IF lp_subset_cnt = 1
	WAIT 'At top level view' WINDOW TIMEOUT 4
ELSE
	lp_subset_cnt = lp_subset_cnt - 1
	lp_more = .T.
	keyboard chr(23)		 && force ctrl-w to end
ENDIF

=brow_key_done()

*******************************************************************************************

PROCEDURE brow_end
*
*	Process the END key
*
=brow_key_start()

STORE "" TO lp_locate, lp_seek  &&   reinitialize locate and seek key
GO BOTTOM			         &&   go to the last record
=brow_key_done()
  
*******************************************************************************************

PROCEDURE brow_home
*
*	Process the HOME key
*
=brow_key_start()

STORE "" TO lp_locate, lp_seek  &&   reinitialize locate and seek key
GO TOP				         &&   go to the first record
=brow_key_done()

*******************************************************************************************

PROCEDURE brow_enter
*
*	Process the ENTER key
*
*	Depending on the configuration that was chosen, the enter key will have
*	one of 4 functions:
*		1. Muiltiple Records - Will toggle the SELECTED field
*		2. 'S' option for pro_browse - Will preserve which field we are resting on.
*				The actual field will only be returned when the 
*				user presses the F2 key.
*		3. Standard - enter key selects record and releases control back 
*			to the calling program.
*		4. 'T' option for pro_browse - select record and DO NOT release
*			control back to the calling program.
*
=brow_key_start()

DO CASE
	CASE selrec	 && ENTER key w/multiple record select
		REPLACE selected WITH !selected	&& Toggle selection
       	IF !EOF()
       		SKIP                            
	       	IF EOF()
       			SKIP -1
       		ENDIF
	       ENDIF
       	lp_seek = ""					 && clear key for next search
     
	CASE AT ('S' , pro_browse) > 0 && Enter
		lp_rec = RECNO()
		
		GOTO lp_rec_sel					&& Turn off last selection
		REPLACE selected WITH .F.

		GOTO lp_rec						&& Turn on new selection
		REPLACE selected WITH .T.
		
		lp_rec_sel = lp_rec
		lp_seek = ''

	CASE AT ('T' , pro_browse) > 0		&& Enter selects but does not
										&& finish browse
		lp_seek=""
	OTHERWISE							&& Standard Enter Key - Select and Return
		pro_ret = .T.
		lp_more = .F.
		keyboard chr(23)			     && force ctrl-w to end
		lp_seek=""                      && clear key for next seek
ENDCASE

=brow_key_done()

*******************************************************************************************

PROCEDURE brow_ESC
*
*	Process the ESC key 
*

=INKEY()
CLEAR TYPEAHEAD

=brow_key_start()

pro_ret = .F.
lp_more    = .F.
keyboard chr(23)

=brow_key_done()

*******************************************************************************************

PROCEDURE brow_F2
*
*	Process the F2 key - Accept
*
*	If not mulitple selection or the pro_browse 'S' option, 
*	treat the same as pressing ENTER
*
=brow_key_start()

DO CASE 
	CASE AT ('T' , pro_browse) > 0
		pro_ret = .T.
		lp_more = .F.
		KEYBOARD CHR (23)
	CASE !(selrec OR AT ('S' , pro_browse) > 0)
		DO brow_enter
	CASE AT ('0' , pro_browse) > 0	&& OK if nothing returned
		pro_ret = .T.
		lp_more = .F.
		KEYBOARD CHR (23)
	OTHERWISE						&& Must have selected a value
		lp_record = RECNO()
		COUNT TO lp_cnt FOR selected
		GOTO lp_record
		IF lp_cnt = 0
			pro_color = 7
			=choose_1 ('Please select items before continuing' , 'OK')
		ELSE
			pro_ret = .T.
			lp_more = .F.
			KEYBOARD CHR (23)                 && force ctrl-w to end
  		ENDIF
ENDCASE

=brow_key_done()

*******************************************************************************************

PROCEDURE brow_back
*
*	Process the backspace key 
*
=brow_key_start()

lp_locate = ""   

* For date fields, skip the '/' when backspacing
IF lp_cur_type = 'D' AND RIGHT (lp_seek,1)="/"  
	lp_seek=left(lp_seek,len(lp_seek)-1)
ENDIF   

* Do the backspace - decrement the string
IF len(lp_seek) > 0
      	lp_seek = LEFT (lp_seek, LEN (lp_seek)-1)
ENDIF   

GO TOP
=findrec()
=brow_key_done()
     
*******************************************************************************************

PROCEDURE brow_insert
*
*	Process the insert key 
*
=brow_key_start()

IF 'P' $ pro_browse		&& Permit add to be run from the active browse
	&insert_proc
ELSE
	addrecrd = .T.
	pro_ret = .T.		&& Assume all OK unless add proc says otherwise
	lp_more = .F.
	keyboard chr(23)	&& force ctrl-w to end browse
ENDIF

=brow_key_done()

*******************************************************************************************

PROCEDURE brow_select

=brow_key_start()

lp_toggle = IIF (selected , 'Deselect Current Record' , 'Select Current Record')
lp_choice = choosev3 ('Select' , 'Select Records' , lp_toggle , 'Select All' , 'Select None')
DO CASE
	CASE lp_choice = 1
		REPLACE selected WITH !selected
	CASE lp_choice = 2
		lp_record = RECNO()
		lp_selection = SUBSTR (lp_subset [lp_subset_cnt] , 5)
		lp_selection = 'FOR !selected ' + IIF (EMPTY (lp_selection) , '' , 'AND ' + lp_selection)
		REPLACE selected WITH .T. &lp_selection
		GOTO lp_record
	CASE lp_choice = 3
		lp_record = RECNO()
		lp_selection = SUBSTR (lp_subset [lp_subset_cnt] , 5)
		lp_selection = 'FOR selected ' + IIF (EMPTY (lp_selection) , '' , 'AND ' + lp_selection)
		REPLACE selected WITH .F. &lp_selection
		GOTO lp_record
ENDCASE

=brow_key_done()

*******************************************************************************************

PROCEDURE brow_key_start

PUSH KEY CLEAR
ON KEY *
ON KEY LABEL LEFTMOUSE

ON KEY LABEL F1 DO helper WITH xz_pro_def [49] 
ON KEY LABEL ALT+F1		DO helpedit WITH xz_pro_def [49] 

*******************************************************************************************

PROCEDURE brow_key_done
*
*	Process common finish for each key
*
DO CASE
	CASE EOF() AND LASTKEY() = 9	&& No match + user backspaced
		lp_locate = ''
		GO TOP
	CASE EOF()						&& No match
		lp_locate = ''
		GO BOTTOM
	CASE BOF()
		lp_locate = ''
		GO TOP
ENDCASE

DO CASE
	CASE lp_nobutton
	CASE (_DOS OR _UNIX) AND EMPTY (lp_wind_brow)
	CASE _DOS OR _UNIX
		ACTIVATE WINDOW pro_stat SAME
		@ lp_notes_offset+3,65 SAY PADR (lp_seek,15) COLOR SCHEME xz_pro_cfg [07]
		ACTIVATE WINDOW (lp_wind_brow) SAME
	OTHERWISE
		SET MESSAGE TO 'Field: ' + PADR (lp_cur_fld , 12) + ;
					   'Search: ' + PADR (lp_seek,15)
ENDCASE
POP KEY
ON KEY LABEL leftmouse DO process_mouse WITH MWINDOW() , MCOL('') , MROW('') , ;
										MCOL (MWINDOW()) , MROW(MWINDOW())

RETURN

**********************************************************************

PROCEDURE mouse_set

* Re-enable the special on-key processing for the mouse
ON KEY LABEL leftmouse DO process_mouse WITH MWINDOW() , MCOL('') , MROW('') , ;
										MCOL (MWINDOW()) , MROW(MWINDOW())

* Give more time before declaring a double-click if we moved into
* a new reocrd
lp_click = SECONDS() - (0.2 * xz_pro_usr [05])
lp_wind_browse = WONTOP()

IF lp_wind_browse <> lp_window
	HIDE WINDOW (lp_window)
ENDIF

RETURN .T.

**********************************************************************
PROCEDURE process_mouse
PARAMETERS m_window , m_x , m_y , m_x_in_wind , m_y_in_wind 

ON KEY LABEL leftmouse		&& The mouse is re-enabled after the
							&& keyboard buffer is emptied

* Clear the keyboard buffer
CLEAR TYPEAHEAD	

* Adjust mouse to coordinates in window
m_in_x = m_x - WLCOL (m_window)
m_in_y = m_y - WLROW (m_window)

*	Check for special regions on the screen being hit

DO CASE

*	->>	Handle the status window
	CASE m_window = 'PRO_STAT'

		=INKEY ('M')			&& Eat the mouse click		
		IF _WINDOWS OR _MAC
			lp_width = 17
			lp_length = 2.15
		ELSE
			ACTIVATE WINDOW (lp_wind_brow) SAME
			lp_width = 15
			lp_length = 3
		ENDIF

		FOR la_cnt = 1 TO 8
			IF BETWEEN ( m_y_in_wind , la_butt [la_cnt , 3] , la_butt [la_cnt , 3] + lp_length) ;
			AND  BETWEEN ( m_x_in_wind , la_butt [la_cnt , 4] , la_butt [la_cnt , 4] + lp_width)
				&la_butt [la_cnt , 2]
				RETURN
			ENDIF
		ENDFOR

*	->> Close box
	CASE (_DOS OR _UNIX) AND ;
		m_y = WLROW (m_window) AND m_x = WLCOL(m_window)
			DO brow_esc
			RETURN

*	->>	Let FOX handle outside of the windows
	CASE m_window <> lp_wind_brow 
*	->> Left of the browsing window.  Window border is a -1.
	CASE m_x_in_wind < 1 
*	->> Top of the browsing window.  Window border is a -1.
	CASE m_y_in_wind < 2 AND (_DOS OR _UNIX)
	CASE m_y_in_wind < 1 AND (_MAC OR _WINDOWS)

*	->> Vertical scroll bar
	CASE m_x_in_wind  > WCOLS (m_window) - lp_factx
*	->> Horizontal scroll bar
	CASE m_y_in_wind  > WROWS (m_window) - lp_facty

*	If multiple records can be selected, test for a double click
	CASE selrec 
		lp_sec = SECONDS()
	
		IF lp_sec - lp_click > xz_pro_usr [05]
			lp_click = lp_sec	&& Reset time to look for the next click
		ELSE
			REPLACE selected WITH !selected	&& Toggle selection
			lp_seek = ""					 	&& Clear key for next search
		ENDIF
		
*	If none of the above, process the mouse click as an 'Enter'
	OTHERWISE
   		DO brow_enter
		RETURN
ENDCASE

* Re-enable the mouse
ON KEY LABEL leftmouse DO process_mouse WITH MWINDOW() , MCOL('') , MROW('') , ;
										MCOL (MWINDOW()) , MROW(MWINDOW())

RETURN

**********************************************************************

PROCEDURE findrec
* if search string is longer than the size of the field being searched,
* we reset the search key to the most recent key hit. For example,
* if the user entered six characters while searching for a 5 character
* account code, we assume that after entering the first code, he
* changed his mind and the keys now hit are a search for a different code.

WAIT "  Searching ...  " WINDOW NOWAIT

ON ESCAPE DO pro_esc		&& Permit user to get out of an escape situation

lp_locate = ''
DO CASE
	CASE lp_cur_type = "C" AND lp_indx_search
   		lp_found = SEEK (lp_seek)     &&   and search again
	CASE lp_cur_type = "C"
   		lp_locate = lp_seek
   		LOCATE FOR UPPER(EVALUATE (lp_cur_fld)) = lp_locate
   		lp_found = FOUND()
   
   CASE lp_cur_type = "N" AND lp_indx_search AND LEN (lp_seek) = lp_cur_len
*        if file is indexed on this numeric key and the whole key
*        has been entered, we can SEEK      
         lp_found = SEEK (VAL(lp_seek))
	CASE lp_cur_type = 'N'
* you can not do a partial search on a numeric key. If user entered
* 7, a seek will find the record CLOSEST to 7. User will want a value
* BEGINNING with 7 (such as 7, 70-79, 700-799, etc.). So if the file
* is indexed and the length of the search key is identical to the length
* of the search field, a seek is done to find the exact record desired.
* If the file is not indexed or the length of the search key is not yet
* equal to the search field, a locate is done to position the pointer
* to the first record that starts with the characters entered.
         lp_locate = lp_seek
         LOCATE FOR LTRIM( STR( EVALUATE(lp_cur_fld))) = lp_locate
         lp_found = FOUND()

	CASE lp_cur_type = "D" AND lp_indx_search AND LEN (lp_seek) = 8
		lp_found = SEEK (CTOD (lp_seek))

	CASE lp_cur_type = "D"
         IF LEN(lp_seek)=2 .OR. LEN(lp_seek)=5
            lp_seek=lp_seek+"/"
         ENDIF

         STORE lp_seek TO lp_locate
         LOCATE FOR DTOC( EVALUATE (lp_cur_fld) )=lp_locate
         lp_found = FOUND()

ENDCASE

ON ESCAPE
WAIT CLEAR

SHOW WINDOW  (lp_wind_brow) REFRESH

RETURN

**********************************************************************

PROCEDURE b_subset

STORE 0 TO lp_search_entire_field , lp_ok , lp_cancel
STORE 1 TO lp_ignore_case

lp_size = MIN (FSIZE (lp_cur_fld) , 60)
lp_srch_for = x2null(EVALUATE (lp_cur_fld))

=pro_win ('w_subset', 'Subset Criteria for ' + lp_cur_fld , 8 , MAX (lp_size + 6,48))

@ 1,2 SAY "Search for:" ;
	SIZE 1,11, 0 ;
	COLOR (xz_pro_def [01])

@ 2,4 GET lp_srch_for ;
	PICTURE '@K' ;
	SIZE 1,lp_size ;
	COLOR (xz_pro_def [01])
		
IF lp_cur_type = 'C' OR lp_cur_type = 'M'
	@ 3,2 GET lp_ignore_case ;
		PICTURE "@*C Ignore Case" ;
		SIZE 1,15 
	@ 4,2 GET lp_search_entire_field ;
		PICTURE "@*C Search Entire Field" ;
		SIZE 1,23
ENDIF

=pro_butt (12 , 5 , 10 , 'lp_ok'     , 'Subset')
=pro_butt (27 , 5 , 10 , 'lp_cancel' , 'Cancel')

ACTIVATE WINDOW w_subset
READ CYCLE MODAL

RELEASE WINDOW w_subset

IF lp_ok = 0 OR EMPTY (lp_srch_for) 	&& Must have been a cancel
	pro_ret = .F.
	RETURN ''
ENDIF

* Construct a search pattern based on the user's request

lp_srch_for = xlate2c (lp_srch_for)

DO CASE
	CASE lp_cur_type = 'C' OR lp_cur_type = 'M'
		lp_srch_for = "'" + ALLTRIM (lp_srch_for) + "'"
		bsub_fld    = lp_cur_fld
		IF lp_ignore_case = 1
			lp_srch_for = "UPPER(" + lp_srch_for + ")"
			bsub_fld    = "UPPER(" + lp_cur_fld + ")"
		ENDIF
		IF lp_search_entire_field = 1
			lp_search = lp_srch_for + ' $ ' + bsub_fld 
		ELSE
			lp_search = bsub_fld + ' = ' + lp_srch_for
		ENDIF
	CASE lp_cur_type = 'N'
		lp_search = lp_cur_fld + ' = ' + lp_srch_for
	CASE lp_cur_type = 'D'
		lp_search = lp_cur_fld + ' = {' + lp_srch_for + '}'
	OTHERWISE
		lp_search = ''
ENDCASE

RETURN lp_search

**********************************************************************

PROCEDURE instruct
* On screen instructions for the user - in the form of push-buttons

* Window for the browser
DO CASE
	CASE lp_nobutton AND WEXIST (lp_window)
		ACTIVATE WINDOW (lp_window)

	CASE lp_nobutton AND (_DOS OR _UNIX)
		DEFINE WINDOW pro_brow ;
			FROM 1,0 TO SROWS() - 1 , SCOLS() -1 ;
			TITLE lp_title ;
			NONE CLOSE NOZOOM NOGROW NOFLOAT NOSHADOW ;
			COLOR (xz_pro_def [4])
		lp_window = 'pro_brow'

	CASE lp_nobutton AND (_WINDOWS OR _MAC)
		DEFINE WINDOW pro_brow ;
			FROM -0.250, 0.000 ;
			TO xz_pro_def [19] , xz_pro_def [20] ;
			TITLE lp_title ;
			FONT xz_pro_def [11] , xz_pro_def [12] ;
			STYLE xz_pro_def [13] ;
			NOFLOAT ;
			CLOSE ;
			NOMINIMIZE ;
			SYSTEM   ;
			COLOR (xz_pro_def [4])
		lp_window = 'pro_brow'
				
	CASE _DOS OR _UNIX
		IF !WEXIST (lp_window)
			DEFINE WINDOW (lp_window) ;
				FROM 0,0 TO lp_message_row-1, lp_scroll_col ;
				TITLE lp_title ;
				NONE CLOSE NOZOOM NOGROW NOFLOAT NOSHADOW ;
				COLOR (xz_pro_def [4])
		ENDIF

		* Window for the bottom of the screen buttons and instructions.
		DEFINE WINDOW pro_stat ;
			FROM lp_message_row,0 TO xz_pro_def [20] - 1 , lp_scroll_col ;
			NONE NOZOOM NOGROW NOFLOAT NOSHADOW ;
			COLOR (xz_pro_def [4])

	CASE _WINDOWS
		* We need to adjust for the size of the screen
		IF !WEXIST (lp_window)
			DEFINE WINDOW (lp_window) ;
				FROM -0.250, 0.000 ;
				TO   lp_message_row , SCOLS()  ;
				TITLE lp_title ;
				FONT xz_pro_def [11] , xz_pro_def [12] ;
				STYLE xz_pro_def [13] ;
				NOFLOAT ;
				CLOSE ;
				NOMINIMIZE ;
				SYSTEM   ;
				COLOR (xz_pro_def [4])
		ENDIF

		DEFINE WINDOW pro_stat ;
			FROM lp_message_row , 0.000 ;
			TO SROWS()  - 0.10 , SCOLS()  ; 
			NOFLOAT ;
			NOCLOSE ;
			NOMINIMIZE ;
			SYSTEM ;
			FONT xz_pro_def [11] , xz_pro_def [12] ;
			STYLE xz_pro_def [13] ;
			COLOR (xz_pro_def [4])
		
		OTHERWISE				&& Must be a Mac

		IF !WEXIST (lp_window)
			DEFINE WINDOW (lp_window) ;
				FROM -0.250, 0.000 ;
				TO   lp_message_row - 3 , SCOLS() - 1 ;
				TITLE lp_title ;
				FONT xz_pro_def [11] , xz_pro_def [12] ;
				STYLE xz_pro_def [13] ;
				NOFLOAT ;
				CLOSE ;
				NOMINIMIZE ;
				SYSTEM   ;
				COLOR (xz_pro_def [4])
		ENDIF

		DEFINE WINDOW pro_stat ;
			FROM lp_message_row - 1 , 0.000 ;
			TO SROWS()  - 2 , SCOLS() - 2  ; 
			NOFLOAT ;
			NOCLOSE ;
			NOMINIMIZE ;
			SYSTEM ;
			FONT xz_pro_def [11] , xz_pro_def [12] ;
			STYLE xz_pro_def [13] ;
			COLOR (xz_pro_def [4])
			
ENDCASE

*	Determine width of scroll bars
IF _WINDOWS OR _MAC
	lp_factx = SYSMETRIC (14) / FONTMETRIC (6, WFONT(1,lp_window),WFONT(2,lp_window),WFONT(3,lp_window)) 
	lp_facty = SYSMETRIC (15) / FONTMETRIC (1, WFONT(1,lp_window),WFONT(2,lp_window),WFONT(3,lp_window)) 
ENDIF

IF lp_nobutton
	RETURN
ENDIF

ACTIVATE WINDOW pro_stat NOSHOW

IF _WINDOWS OR _MAC

	lp_row = lp_notes_offset + 0.15
	lp_col = 0.15

	=butt_pos(1)				&& Place the accept button
	=butt_pos(2)				&& Place the cancel button
	
	IF !('5' $ pro_browse)
		=butt_pos(3)			&& Search
		=butt_pos(4)			&& Search again
	ENDIF
	IF !('6' $ pro_browse)
		=butt_pos(5)			&& Subset
		=butt_pos(6)			&& Superset
	ENDIF
	IF !EMPTY (insert_proc)	&& Special instructions if add record capability
		=butt_pos(7)			&& Insert proc
	ENDIF

	IF selrec			&& Special instructions if multiple record selection
		=butt_pos(8)
	ENDIF

	IF lp_notes_offset > 0	&& Add special notes to screen
		lp_notes = ALLTRIM (lp_notes)
		@ 1, 0.5 * (WCOLS() - TXTWIDTH (lp_notes)) SAY lp_notes	 && Special text
	ENDIF

	ACTIVATE WINDOW pro_stat SAME
	CLEAR GETS				&& We really only wanted the button pictures on screen

	RETURN

ENDIF						&& Must be DOS

* Set rows for each button into the la_butt array
lp_row1 = lp_notes_offset 
lp_row2 = lp_row1 + 3.15
STORE lp_row1 -1 TO la_butt [1,3] , la_butt [2,3] , la_butt [7,3] , la_butt [8,3]
STORE lp_row2 -1 TO la_butt [3,3] , la_butt [4,3] , la_butt [5,3] , la_butt [6,3]

la_butt [1,4] = 0
=pro_butt (0 ,lp_row1 ,15,'l_accept', PADR (la_butt [1,1],10) + ' F2')	&& Accept

la_butt [2,4] = 14
=pro_butt (15,lp_row1,15,'l_accept', PADR (la_butt [2,1],10) + 'Esc')	&& Cancel

IF !('5' $ pro_browse)
	la_butt [3,4] = 0
	la_butt [4,4] = 14
	=pro_butt (0 ,lp_row2,15,'l_accept',PADR (la_butt [3,1],10) + ' F5')	&& Search
	=pro_butt (15,lp_row2,15,'l_accept',PADR (la_butt [4,1],7) + 'Alt-F5')	&& Again
ENDIF

IF !('6' $ pro_browse)
	la_butt [5,4] = 29
	la_butt [6,4] = 44
	=pro_butt (30,lp_row2,15,'l_accept',PADR (la_butt [5,1],10) + ' F6')	&& SubSet
	=pro_butt (45,lp_row2,15,'l_accept',PADR (la_butt [6,1],7) + 'Alt-F6')	&& SuperSet
ENDIF

IF !EMPTY (insert_proc)	&& Special instructions if add record capability
	la_butt [7,4] = 44
	=pro_butt (45,lp_row1,15,'l_accept',PADR (la_butt [7,1],10) + 'Ins')	&& Add
ENDIF

@ lp_notes_offset,63 SAY "Search field:" COLOR (xz_pro_def [01])
@ lp_notes_offset+2,63 SAY "Search for:"   COLOR (xz_pro_def [01])

IF selrec			&& Special instructions if multiple record selection
	la_butt [8,4] = 29
	=pro_butt (30,lp_row1,15,'l_accept',PADR (la_butt [8,1],8) + 'Enter')	&& Select
	@ lp_row1+4,63 SAY "Multiple Select" COLOR (xz_pro_def [01])
ELSE
	@ lp_row1+4,63 SAY "Single Selection" COLOR (xz_pro_def [01])
ENDIF

IF lp_notes_offset > 0	&& Add special notes to screen
	@ 1,1 SAY PADC (lp_notes , SCOLS()) && Special text
	@ 0,0 TO lp_notes_offset -1 , SCOLS()-1		&& Box
ENDIF

ACTIVATE WINDOW pro_stat SAME
CLEAR GETS				&& We really only wanted the button pictures on screen

RETURN

**********************************************************************

FUNCTION butt_pos
PARAMETER butt_element

la_butt [butt_element , 3] = lp_row
la_butt [butt_element , 4] = lp_col

@ lp_row , lp_col GET l_accept ;
	PICTURE "@*HN " + la_butt [butt_element , 1] ;
	SIZE xz_pro_def [14] ,17,0.500 ;
	DEFAULT 1 ;
	FONT xz_pro_def [11] , xz_pro_def [12] ;
	STYLE xz_pro_def [13] ;
	COLOR (xz_pro_def [01])

lp_col = lp_col + 17
IF lp_col + 17 > WCOLS()
	lp_col = 0.15
	lp_row = lp_row + xz_pro_def [14] + 0.15
ENDIF

RETURN

**********************************************************************

FUNCTION b_insert_when
PARAMETERS lp_fields
PRIVATE ALL LIKE 'lp_*'
*
* Find each field.  Add a WHEN clause to permit gauranteed test
* of entry into a field.  If there already was a WHEN clause,
* process that statement AFTER the field information is extracted.
*
*
* Fix potential programmer entry errors when calling
* this routine.  If '|' is not specified, replace all commas with '|'.
lp_field_rem = IIF (AT ('|' , lp_fields) = 0 , ;
					STRTRAN (lp_fields , ',' , '|') , ;
					lp_fields)

STORE '' TO lp_token , lp_fields , lp_sep

* Scan for each field (separated by the '|' or terminated by end of line)
DO WHILE parse (@lp_field_rem , '|' , .T. , @lp_token , @lp_sep)

	lp_pos_when  = ATC(':W', lp_token)
	IF lp_pos_when > 0			&& Existing WHEN field
	
		* Extract items up to the :W clause
		lp_fields = lp_fields + ',' + LEFT (lp_token , lp_pos_when - 1)
		
		* Extract from the :W portion of the clause
		lp_when = LTRIM (SUBSTR (lp_token , lp_pos_when + 2))
		
		* Remove the equal sign following the when
		lp_equal= AT ('=' , lp_when)
		lp_when = LTRIM (SUBSTR (lp_when , lp_equal + 1))
		
		* Locate the end of the WHEN clause
		lp_end = ATC (':' , lp_when)
		
		IF lp_end = 0		&& Was this the last portion of the instruction?
			lp_fields = lp_fields + ':W=b_w("' + lp_when + '")'
		ELSE
			lp_fields = lp_fields + ;
						':W=b_w("' + LEFT (lp_when , lp_end - 1) + '")' + ;
						SUBSTR (lp_when , lp_end)
		ENDIF
		
	ELSE
		lp_fields = lp_fields + ',' + lp_token + ':W=b_w()'
	ENDIF		

ENDDO

RETURN SUBSTR (lp_fields , 2)

**********************************************************************

FUNCTION b_w
PARAMETER orig_when

lp_when = IIF (PARAMETERS()>0 , EVALUATE (orig_when) , .T.)

testFld = UPPER (VARREAD())
varReadType = TYPE (testFld)

* Did field change
IF testfld == lp_cur_fld
	RETURN lp_when
ENDIF

STORE "" to lp_locate, lp_seek   && deactivate locate scope
lp_cur_fld = testFld
lp_cur_type = varReadType
   
DO CASE
	CASE varReadType = "C"
		lp_cur_len = FSIZE (lp_cur_fld)
	CASE varReadType = "D"
		lp_cur_len = 8
	CASE varReadType = "U"
		lp_cur_len = 0
    OTHERWISE				&& must be a number
    	lp_cur_len = 6
    ENDCASE
    
lp_indx_search = (lp_indx == lp_cur_fld)	&& Can we do indexed searches

DO CASE
	CASE lp_nobutton
	CASE _DOS
		ACTIVATE WINDOW pro_stat SAME
		IF lp_indx_search
			@ lp_notes_offset+1,64 SAY '*' COLOR SCHEME xz_pro_cfg [07]
		ELSE
			@ lp_notes_offset+1,64 SAY ' ' COLOR SCHEME xz_pro_cfg [07]
		ENDIF
		@ lp_notes_offset+1,65 SAY PADR (lp_cur_fld, 10)	COLOR SCHEME xz_pro_cfg [07]
		@ lp_notes_offset+3,65 SAY PADR (lp_seek,15)		COLOR SCHEME xz_pro_cfg [07]
		ACTIVATE WINDOW (lp_wind_brow) SAME
	OTHERWISE
		SET MESSAGE TO 'Field: ' + ;
						IIF (lp_indx_search , '*' , ' ') + ;
						PADR (lp_cur_fld , 12) + ;
					   'Search: ' + PADR (lp_seek,15)
ENDCASE

* Re-enable the mouse
ON KEY LABEL leftmouse DO process_mouse WITH MWINDOW() , MCOL('') , MROW('') , ;
										MCOL (MWINDOW()) , MROW(MWINDOW())

RETURN lp_when

